#!/usr/bin/perl
##!/usr/bin/perl -Tw
#This is IP_AUTH_LDAP auth.cgi based on IP_AUTH_HACK by <steffen@dett.de>

use strict;
use CGI;

# Debugging
my $DEBUG=0;

$ENV{'PATH'}='/bin:/usr/bin';
my $IP_AUTH_LIB='/usr/lib/squid/squid_ip_auth/';

# Looks configuration
my $header_file="/var/www/squid_ip_auth/html/header.html";
my $footer_file="/var/www/squid_ip_auth/html/footer.html";
my $css_file="https://proxyauth.fct.unl.pt/css/css.html";
my $lang="pt"; #only "pt", "en" and "de" supported, all others fallback to "en"

# Common configuration
# AUTH_SCHEMA only "ldap" is supported for now
my $AUTH_SCHEMA="ldap";

# Usually there's no need to edit beyond this point
my $fifo_file = "/var/run/squid/ip_auth.fifo";

my $cgi=new CGI;

my $lang_file=$IP_AUTH_LIB."ip_auth-lang.inc.pl";
my $auth_schema_file=$IP_AUTH_LIB."ip_auth-".$AUTH_SCHEMA.".inc.pl";

if (-r $auth_schema_file) {
	require $auth_schema_file;
} else {
	print $cgi->center($cgi->h1("Authentication Method not Suported!"));
	print $cgi->center($cgi->h3("Check you CGI AUTH_SCHEMA configuration!"));
	die "Bad Auth Method - see AUTH_SCHEMA" ;
}

require $lang_file;

my %mess = set_lang($lang);

sub red(@)
{
	if ($#_ == 0) {
		return ('<FONT COLOR="RED">' . $_[0] . '</FONT>');
	} else {
		return ('<FONT COLOR="RED">' , @_ , '</FONT>');
	}
}

sub print_header() {

	print $cgi->start_html(-title=>$mess{'title'}, -bgcolor=>"white", -style=>{'src'=>$css_file}), "\n";

	if (open(HEADER, "<$header_file")) {
		while (<HEADER>) {
			print $_;
		}
	}
	else {
		# Do nothing. Header file simply doesn't exist.
	}
	close HEADER;

}

sub print_footer() {

	if (open(FOOTER, "<$footer_file")) {
		while (<FOOTER>) {
			print $_;
		}
	}
	else {
		# Do nothing. Footer file simply doesn't exist.
	}
	close FOOTER;

	print $cgi->end_html;

}

sub print_fifo(@)
{
	if (!open(PIPE, ">>$fifo_file")) {
		print $cgi->h1(red($mess{'fifo_err'})), "(open)\n";
		print_footer();
		die ("$0: FIFO open failed [$!]");
	}
	if (!print(PIPE @_)) {
		print $cgi->h1(red($mess{'fifo_err'})), "(write)\n";
		print_footer();
		die ("$0: FIFO write failed [$!]");
	}
	if (!close(PIPE)) {
		print $cgi->h1(red($mess{'fifo_err'})), "(close)\n";
		print_footer();
		die ("$0: FIFO close failed [$!]");
	}
}


sub check_connect($) {

	my $url = shift;

	if ($url =~ m|CONNECT+://| ) {
		return undef;
	} else {
		return 1;
	}
}


sub fix_https($) {

	my $url = shift;

	if (!($url =~ m|^http:\/\/|) && (($url =~ m|:443$|) || ($url =~ m|:10000$|))) {
		$url = "https://".$url;
	}
	return $url;
}

#IP Address to authenticate
my $addr = $ENV{'REMOTE_ADDR'};
my $url = $cgi->param('URL');
my $user = $cgi->param('login');
my $pass = $cgi->param('pass');

$url = fix_https($url);

#If we had a CONNECT (SSL), we don't have a valid URL (since
#  the Browser sent "CONNECT <server>:<port>" )
if (check_connect($url)) {
	print $cgi->header(), "\n";
} else {
	print $cgi->header(-Refresh=>"1; URL=$url"), "\n";
}

print_header();

if ($DEBUG==1) {

	print "\n\n<!-- Infotable -->\n";
	print $cgi->table({-border=>"0", -width=>"95%", -align=>"center"},
		$cgi->Tr( $cgi->td( $cgi->table({-border=>"0", -cellpadding=>"10", width=>"75%"},
		$cgi->caption($mess{'login_info'}."\n"),
		$cgi->Tr({-align=>"CENTER",-valign=>"TOP"},
			[
			$cgi->th( [ $mess{'info'}, $mess{'val'}, $mess{'src'}."\n" ] ),
			$cgi->td( [ $mess{'user'}, red($user)."&nbsp;", "Login\n" ] ),
			$cgi->td( [ $mess{'pass'}, $pass . "&nbsp;", "Login\n" ] ),
			$cgi->td( [ $mess{'addr'}, red($addr)."&nbsp;", $mess{'conn'}."\n" ] ),
			$cgi->td( [ $mess{'url'}, $url, $mess{'proxy'} . "\n" ] ),
			] )
		) ) )
	);
	print "\n<!-- Infotable end -->\n\n";

}

#Non-buffered Mode from now:
$| = 1; print "\n\n";

if (verify_user($user, $pass)) {

	print_fifo("+$addr");
	print $cgi->center($cgi->h3($mess{'login_ok'}));
	print_footer();
	sleep 2; #delay...

} else {

	print $cgi->center($cgi->h1(red($mess{'error'}), $mess{'incorrect'}));
	print $cgi->center($cgi->h3($mess{'fail'}));
	print $cgi->center($cgi->h3($mess{'wait_again'}));
	print_fifo("-$addr");
	print_footer();
	sleep 3; #delay...
}

if (check_connect($url)) {
	# Known possible hack, but i'm no html expert
	print '<META HTTP-EQUIV="Refresh" CONTENT="0; URL='.$url.'">', "\n";
}
